@@ -1,7 +1,7 @@
 /* coverage.c -- samtools coverage subcommand
 
     Copyright (C) 2018,2019 Florian Breitwieser
-    Portions copyright (C) 2019-2021, 2023 Genome Research Ltd.
+    Portions copyright (C) 2019-2021, 2023-2025 Genome Research Ltd.
 
     Author: Florian P Breitwieser <florian.bw@gmail.com>
 
@@ -60,6 +60,7 @@ typedef struct {  // auxiliary data structure to hold stats on coverage
     unsigned long long summed_coverage;
     unsigned long long summed_baseQ;
     unsigned long long summed_mapQ;
+    unsigned long long quality_bases;
     unsigned int n_reads;
     unsigned int n_selected_reads;
     bool covered;
@@ -105,7 +106,7 @@ static const char *const BLOCK_CHARS2[2] = {".", ":"};
 // in bam_plcmd.c
 int read_file_list(const char *file_list, int *n, char **argv[]);
 
-static int usage() {
+static int usage(void) {
     fprintf(stdout, "Usage: samtools coverage [options] in1.bam [in2.bam [...]]\n\n"
             "Input options:\n"
             "  -b, --bam-list FILE     list of input BAM filenames, one per line\n"
@@ -118,6 +119,8 @@ static int usage() {
             "  -d, --depth INT         maximum allowed coverage depth [1000000].\n"
             "                          If 0, depth is set to the maximum integer value,\n"
             "                          effectively removing any depth limit.\n"
+            "      --min-depth INT     minimum coverage depth below which a position \n"
+            "                          to be ignored [1]\n"
             "Output options:\n"
             "  -m, --histogram         show histogram instead of tabular output\n"
             "  -D, --plot-depth        plot depth instead of tabular output\n"
@@ -194,7 +197,15 @@ static int read_bam(void *data, bam1_t *b) {
     return ret;
 }
 
-void print_tabular_line(FILE *file_out, const sam_hdr_t *h, const stats_aux_t *stats, int tid) {
+void print_tabular_line(FILE *file_out, const sam_hdr_t *h, const stats_aux_t *stats, int tid,
+    bool *header) {
+
+    if (*header) {
+        fputs("#rname\tstartpos\tendpos\tnumreads\tcovbases\tcoverage\tmeandepth\tmeanbaseq\tmeanmapq\n",
+         file_out);
+        *header = false;
+    }
+
     fputs(sam_hdr_tid2name(h, tid), file_out);
     double region_len = (double) stats[tid].end - stats[tid].beg;
     fprintf(file_out, "\t%"PRId64"\t%"PRId64"\t%u\t%llu\t%g\t%g\t%.3g\t%.3g\n",
@@ -204,7 +215,7 @@ void print_tabular_line(FILE *file_out, const sam_hdr_t *h, const stats_aux_t *s
             stats[tid].n_covered_bases,
             100.0 * stats[tid].n_covered_bases / region_len,
             stats[tid].summed_coverage / region_len,
-            stats[tid].summed_coverage > 0? stats[tid].summed_baseQ/(double) stats[tid].summed_coverage : 0,
+            stats[tid].quality_bases > 0? stats[tid].summed_baseQ/(double) stats[tid].quality_bases : 0,
             stats[tid].n_selected_reads > 0? stats[tid].summed_mapQ/(double) stats[tid].n_selected_reads : 0
            );
 }
@@ -263,7 +274,7 @@ void print_hist(FILE *file_out, const sam_hdr_t *h, const stats_aux_t *stats, in
             case 5: fprintf(file_out, "Mean coverage:   %.3gx",
                             stats[tid].summed_coverage / region_len); break;
             case 4: fprintf(file_out, "Mean baseQ:      %.3g",
-                            stats[tid].summed_baseQ/(double) stats[tid].summed_coverage); break;
+                            stats[tid].quality_bases > 0? stats[tid].summed_baseQ/(double) stats[tid].quality_bases : 0); break;
             case 3: fprintf(file_out, "Mean mapQ:       %.3g",
                             stats[tid].summed_mapQ/(double) stats[tid].n_selected_reads); break;
             case 1: fprintf(file_out, "Histo bin width: %sbp",
@@ -315,7 +326,8 @@ int main_coverage(int argc, char *argv[]) {
     char **fn = NULL;
     int fail_flags = (BAM_FUNMAP | BAM_FSECONDARY | BAM_FQCFAIL | BAM_FDUP); // Default fail flags
     int required_flags = 0;
-
+    int print_value_warning = 0;
+    int mindepth = 1;   //min depth in file, below which pos is ignored
     int *n_plp = NULL;
     sam_hdr_t *h = NULL; // BAM header of the 1st input
 
@@ -349,6 +361,7 @@ int main_coverage(int argc, char *argv[]) {
         {"region", required_argument, NULL, 'r'},
         {"help", no_argument, NULL, 'h'},
         {"depth", required_argument, NULL, 'd'},
+        {"min-depth", required_argument, NULL, 3},
         { NULL, 0, NULL, 0 }
     };
 
@@ -365,6 +378,11 @@ int main_coverage(int argc, char *argv[]) {
                 if ((fail_flags = bam_str2flag(optarg)) < 0) {
                     fprintf(stderr,"Could not parse --ff %s\n", optarg); return EXIT_FAILURE;
                 }; break;
+            case 3: //min-depth
+                if ((i = atoi(optarg)) > 0) {
+                    mindepth = i;
+                }
+                break;
             case 'o': opt_output_file = optarg; opt_full_width = false; break;
             case 'l': opt_min_len = atoi(optarg); break;
             case 'q': opt_min_mapQ = atoi(optarg); break;
@@ -522,9 +540,6 @@ int main_coverage(int argc, char *argv[]) {
         }
     }
 
-    if (opt_print_tabular && opt_print_header)
-        fputs("#rname\tstartpos\tendpos\tnumreads\tcovbases\tcoverage\tmeandepth\tmeanbaseq\tmeanmapq\n", file_out);
-
     h = data[0]->hdr; // easy access to the header of the 1st BAM
     int n_targets = sam_hdr_nref(h);
     stats = calloc(n_targets, sizeof(stats_aux_t));
@@ -570,6 +585,7 @@ int main_coverage(int argc, char *argv[]) {
         status = EXIT_FAILURE;
         goto coverage_end;
     }
+
     while ((ret=bam_mplp_auto(mplp, &tid, &pos, n_plp, plp)) > 0) { // come to the next covered position
 
         if (tid != old_tid) { // Next target sequence
@@ -578,7 +594,7 @@ int main_coverage(int argc, char *argv[]) {
                     print_hist(file_out, h, stats, old_tid, hist, n_bins, opt_full_utf, opt_plot_coverage);
                     fputc('\n', file_out);
                 } else if (opt_print_tabular) {
-                    print_tabular_line(file_out, h, stats, old_tid);
+                    print_tabular_line(file_out, h, stats, old_tid, &opt_print_header);
                 }
 
                 if (opt_print_histogram)
@@ -604,33 +620,51 @@ int main_coverage(int argc, char *argv[]) {
         }
 
         bool count_base = false;
+        unsigned long long summed_baseQ = 0, quality_bases = 0, depth = 0;
         for (i = 0; i < n_bam_files; ++i) { // base level filters have to go here
             int depth_at_pos = n_plp[i];
             for (j = 0; j < n_plp[i]; ++j) {
                 const bam_pileup1_t *p = plp[i] + j; // DON'T modify plp[][] unless you really know
 
-                if (p->is_del || p->is_refskip) --depth_at_pos; // having dels or refskips at tid:pos
-                else if (p->qpos < p->b->core.l_qseq &&
-                        bam_get_qual(p->b)[p->qpos] < opt_min_baseQ) --depth_at_pos; // low base quality
-                else
-                    stats[tid].summed_baseQ += bam_get_qual(p->b)[p->qpos];
+                if (p->is_del || p->is_refskip) {
+                    --depth_at_pos; // having dels or refskips at tid:pos
+                } else if (p->qpos < p->b->core.l_qseq) {
+                    if (bam_get_qual(p->b)[p->qpos] < opt_min_baseQ) {
+                        --depth_at_pos; // low base quality
+                    } else {
+                        summed_baseQ += bam_get_qual(p->b)[p->qpos];
+                        ++quality_bases;
+                    }
+                } else {
+                    print_value_warning = 1; // no quality at position
+                }
             }
+
             if (depth_at_pos > 0) {
                 count_base = true;
-                stats[tid].summed_coverage += depth_at_pos;
+                depth += depth_at_pos;
             }
 
             if(current_bin < n_bins && opt_plot_coverage) {
                 hist[current_bin] += depth_at_pos;
-                }
+            }
         }
-        if (count_base) {
+        if (count_base && depth >= mindepth) {
+            stats[tid].summed_coverage += depth;
+            stats[tid].summed_baseQ += summed_baseQ;
+            stats[tid].quality_bases += quality_bases;
+
             stats[tid].n_covered_bases++;
             if (opt_print_histogram && current_bin < n_bins && !opt_plot_coverage)
                 ++(hist[current_bin]); // Histogram based on breadth of coverage
         }
     }
 
+    if (ret < 0) {
+        status = EXIT_FAILURE;
+        goto coverage_end;
+    }
+
     if (tid == -1 && opt_reg && *opt_reg != '*')
         // Region specified but no data covering it.
         tid = data[0]->iter->tid;
@@ -639,7 +673,7 @@ int main_coverage(int argc, char *argv[]) {
         if (opt_print_histogram) {
             print_hist(file_out, h, stats, tid, hist, n_bins, opt_full_utf, opt_plot_coverage);
         } else if (opt_print_tabular) {
-            print_tabular_line(file_out, h, stats, tid);
+            print_tabular_line(file_out, h, stats, tid, &opt_print_header);
         }
     }
 
@@ -648,12 +682,15 @@ int main_coverage(int argc, char *argv[]) {
         for (i = 0; i < n_targets; ++i) {
             if (!stats[i].covered) {
                 stats[i].end = sam_hdr_tid2len(h, i);
-                print_tabular_line(file_out, h, stats, i);
+                print_tabular_line(file_out, h, stats, i, &opt_print_header);
             }
         }
     }
 
-    if (ret < 0) status = EXIT_FAILURE;
+    if (print_value_warning) {
+        print_error("coverage", "Warning:  Missing quality values in alignments.  Mean base quality calculated only on available values.");
+    }
+
 
 coverage_end:
     if (n_plp) free(n_plp);
